@@ -1,34 +1,36 @@
 package Proc::Queue;
 
 require 5.005;
+
+our $VERSION = '1.15';
+
 use strict;
 # use warnings;
 require Exporter;
 use Carp;
-
 use POSIX ":sys_wait_h";
 
 our @ISA = qw(Exporter);
-
 our %EXPORT_TAGS = ( all => [ qw( fork_now
-				    waitpids
-				    run_back
-				    run_back_now
-				    system_back
-				    system_back_now
-				    all_exit_ok
-				    running_now ) ] );
-
+				  waitpids
+				  run_back
+				  run_back_now
+				  system_back
+				  system_back_now
+				  all_exit_ok
+				  running_now ) ] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
 
-our $VERSION = '1.13';
+# are we running on Windows?
+use constant _win => $^O=~/win32/i;
 
 # parameters
 my $queue_size=4; # max number of councurrent processes running.
 my $debug=0; # shows debug info.
-my $trace=0; # shows function enters.
+my $trace=0; # shows function calls.
 my $delay=0; # delay between fork calls.
+my $ignore_childs=0; # similar to $SIG{CHILD}='IGNORE';
 
 my $last=0; # last time fork was called.
 
@@ -44,12 +46,13 @@ my @captured;
 sub import {
   my ($pkg,@opts)=@_;
   my $i;
-  for ($i=0; $i<=$#opts; $i++) {
+  for ($i=0; $i<@opts; $i++) {
     my $o=$opts[$i];
     if( $o eq 'size'
         or $o eq 'debug'
         or $o eq 'trace'
-	or $o eq 'delay' ) {
+	or $o eq 'delay'
+	or $o eq 'ignore_childs') {
       $#opts>$i or croak "option '$o' needs a value";
       my $value=$opts[$i+1];
       { no strict qw( subs refs );
@@ -63,63 +66,70 @@ sub import {
 }
 
 sub delay {
-  my $d=shift;
+  return $delay unless @_;
   my $old_delay=$delay;
-  if(defined $d) {
-    $delay=$d;
-    carp "Proc queue delay set to $delay sec., it was $old_delay" if $debug;
-  }
+  $delay=$_[0];
+  carp "Proc queue delay set to ${delay}s, it was $old_delay" if $debug;
   return $old_delay;
 }
 
 sub size {
+  return $queue_size unless @_;
   my $size=shift;
   my $old_size=$queue_size;
-  if(defined $size) {
-    croak "invalid value for Proc::Queue size ($size), min value is 1"
-      unless $size >= 1;
-    $queue_size=$size;
-    carp "Proc queue size set to $queue_size, it was $old_size" if $debug;
-  }
+  croak "invalid value for Proc::Queue size ($size), min value is 1"
+    unless $size >= 1;
+  $queue_size=$size;
+  carp "Proc queue size set to $size, it was $old_size" if $debug;
   return $old_size;
 }
 
 sub debug {
-  my $d=shift;
+  return $debug unless @_;
   my $old_debug=$debug;
-  if(defined $d) {
-    if ($d) {
-      $debug=1;
-      carp "Debug mode is now ON for Proc::Queue module";
-    }
-    else {
-      $debug=0;
-      carp "Debug mode is now OFF for Proc::Queue module" if $old_debug;
-    }
+  if ($_[0]) {
+    $debug=1;
+    carp "debug mode ON";
+  }
+  else {
+    $debug=0;
+    carp "debug mode OFF" if $old_debug;
   }
   return $old_debug;
 }
 
 sub trace {
-  my $t=shift;
+  return $trace unless @_;
   my $old_trace=$trace;
-  if(defined $t) {
-    if ($t) {
-      $trace=1;
-      carp "Trace mode is now ON for Proc::Queue module" if $debug;
-    }
-    else {
-      $trace=0;
-      carp "Trace mode is now OFF for Proc::Queue module" if $debug;
-    }
+  if ($_[0]) {
+    $trace=1;
+    carp "trace mode ON" if $debug;
+  }
+  else {
+    $trace=0;
+    carp "trace mode OFF" if $debug;
   }
   return $old_trace;
 }
 
+sub ignore_childs {
+  return $ignore_childs unless @_;
+  my $old_ignore_childs=$ignore_childs;
+  if ($_[0]) {
+    $ignore_childs=1;
+    carp "ignore_childs mode ON" if $debug;
+  }
+  else {
+    $ignore_childs=0;
+    carp "ignore_childs mode OFF" if $debug;
+  }
+  return $old_ignore_childs;
+}
+
 # sub to store internally captured processes
 sub _push_captured {
-  push @captured, shift,$?;
-  croak "captured stack is corrupted" unless ($#captured & 1)
+  push @captured, shift,$? unless $ignore_childs;
+  croak "captured stack is corrupted" if (@captured & 1)
 }
 
 # do the real wait and housekeeping
@@ -148,7 +158,7 @@ sub new_wait () {
   if(@captured) {
     my $w=shift @captured;
     $?=shift @captured;
-    carp "Wait returning old child $w captured in fork" if $debug;
+    carp "Wait returning old child $w captured in wait" if $debug;
     return $w;
   }
   return _wait;
@@ -177,7 +187,11 @@ sub _waitpid ($$) {
 
 sub _clean() {
   my $pid;
-  _push_captured($pid) while(($pid=_waitpid(-1,WNOHANG))>0);
+  while (1) {
+    $pid=_waitpid(-1,WNOHANG);
+    return unless ((_win && $pid < -1) || $pid >0);
+    _push_captured $pid
+  }
 }
 
 sub new_waitpid ($$) {
@@ -208,7 +222,7 @@ BEGIN { eval "use Time::HiRes 'time'" }
 sub _fork () {
   carp "Proc::Queue::_fork called" if $trace && $debug;
   if ($delay>0) {
-    my $wait=$last+$delay - time();
+    my $wait=$last+$delay - time;
     if ($wait>0) {
       carp "Delaying $wait seconds before forking" if $debug;
       select (undef,undef,undef,$wait);
@@ -240,7 +254,7 @@ sub _fork () {
 sub new_fork () {
   carp "Proc::Queue::fork called" if $trace;
   while($queue_now>=$queue_size) {
-    carp "Waiting that some process finishes before continuing" if $debug;
+    carp "Waiting for some process to finish" if $debug;
     my $nw;
     if (($nw=_wait) != -1) {
       _push_captured $nw;
@@ -250,7 +264,7 @@ sub new_fork () {
       last;
     }
   }
-  return _fork();
+  return _fork;
 }
 
 sub fork_now () {
@@ -265,7 +279,7 @@ sub waitpids {
     if (defined($pid)) {
       carp "Waiting for child $pid to exit" if $debug;
       my $r=new_waitpid($pid,0);
-      if ($r > 0) {
+      if ((_win && $r < -1) || $r > 0) {
 	carp "Child $r return $?" if $debug;
 	push @result,$r,$?;
       }
@@ -343,10 +357,10 @@ sub all_exit_ok {
   carp "Proc::Queue::all_exit_ok(".join(", ",@_).")" if $trace;
   my @result=waitpids(@_);
   my $i;
-  for($i=0;$i<=$#result;$i++) {
+  for($i=0;$i<@result;$i++) {
     next unless $i&1;
     if (!defined($result[$i]) or $result[$i]) {
-      carp "Child $_[$i>>1] fail with code $result[$i], waitpid return $result[$i-1]" if $debug;
+      carp "Child ".$_[$i>>1]." fail with code $result[$i], waitpid return $result[$i-1]" if $debug;
       return 0;
     }
   }
@@ -437,7 +451,7 @@ that seem too be very useful when developing parallel aplications:
 =item debug mode:
 
 when active, dumps lots of information about processes being created,
-exiting, being caught be parent, etc.
+exiting, being caught by parent, etc.
 
 =item trace mode:
 
@@ -454,12 +468,20 @@ queues are reset and the maximun process number for them is set to 1
 (anyway, childs can change their queue size themselves).
 
 Proc::Queue doesn't work if CHLD signal handler is set to
-C<IGNORE>.
+C<IGNORE>.  
 
 Internally, Proc::Queue, automatically catches zombies and stores
 their exit status in a private hash. To avoid leaking memory in long
 running programs you have to call C<wait> or C<waitpid> to delete
-entries from that hash.
+entries from that hash or alternatively active the C<ignore_childs>
+mode:
+
+  Proc::Queue::ignore_childs(1)
+
+or
+
+  use Proc::Queue ignore_childs=>1, ...
+
 
 =head2 EXPORT
 
@@ -500,6 +522,18 @@ If no arg is given, the current delay is returned.
 
 To clear it use C<Proc::Queue::delay(0)>.
 
+=item ignore_childs($on)
+
+calling
+
+  Proc::Queue::ignore_childs(1);
+
+is the equivalent to
+
+  $SIG{CHLD}='IGNORE'
+
+when using Proc::Queue.
+
 =item debug(), debug($boolean), trace(), trace($boolean)
 
 Change or return the status for the debug and trace modes.
@@ -576,9 +610,10 @@ function name to be imported.
 
 =head2 BUGS
 
-Proc::Queue is a very stable module, and no bugs have been reported
+Proc::Queue is a very stable module, no bugs have been reported
 for a long time.
 
+Support for Win32 OSs is still experimental.
 
 =head1 SEE ALSO
 
@@ -589,7 +624,7 @@ contained in the module distribution.
 
 =head1 AUTHOR
 
-Salvador Fandio <sfandino@yahoo.com>
+Salvador Fandio E<lt>sfandino@yahoo.comE<gt>
 
 
 =head1 COPYRIGHT AND LICENSE
